package org.example;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.Calendar;
import java.util.Random;

public class DonorsGenerator {
    static final String[] CITIES = {"Montreal", "Laval", "Brossard"};
    static final String[] GENDERS = {"M", "F", "U"};

    private static final String[] STREET_NAMES = {
            "Pine", "Oak", "Lakeview", "Sunset", "Forest", "River", "Spring",
            "Main", "First", "Wind", "Park", "Maple", "Elm", "Cerulean"
    };
    protected static final String[] HOBBIES = {
            "Hiking", "Skiing", "Swimming", "Bird watching", "Racing", "Walking", "Gaming",
    };

    private static Random RANDOM;

    private static Connection conn;

    private final int count;

    public DonorsGenerator(int recordCount, Connection conn, Random random) {
        DonorsGenerator.conn = conn;
        count = recordCount;
        DonorsGenerator.RANDOM = random;
    }

    public void insertDonors() throws SQLException {
        String sql = "INSERT INTO Donors (donorID, firstName, lastName, middleInitial, dateOfBirth, address, city, postalCode, province, gender, SSN, hobby, phone, email) " +
                "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";

        PreparedStatement pstmt = conn.prepareStatement(sql);

        for (int i = 1; i <= count; i++) {
            pstmt.setInt(1, i);  // donorID
            pstmt.setString(2, randomString(5));
            pstmt.setString(3, randomString(7));
            pstmt.setString(4, randomInitial());
            pstmt.setDate(5, randomDateOfBirth());
            pstmt.setString(6, randomAddress());
            pstmt.setString(7, randomCities());  // city (static data for simplicity)
            pstmt.setString(8, randomPostalCode());  // postalCode (static data for simplicity)
            pstmt.setString(9, "QC");  // province
            pstmt.setString(10, randomGender());  // gender
            pstmt.setString(11, randomSSN());  // SSN
            pstmt.setString(12, randomHobbies());  // hobby (static data for simplicity)
            pstmt.setString(13, "1234567890");  // phone (static data for simplicity)
            pstmt.setString(14, "email" + i + "@example.com");  // email (static data for simplicity)

            pstmt.addBatch();
        }

        pstmt.executeBatch();  // Execute all inserts as a batch
        pstmt.close();
    }

    private static String randomString(int length) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < length; i++) {
            char c = (char) (RANDOM.nextInt(26) + 'a');
            sb.append(c);
        }
        return sb.toString();
    }

    private static String randomInitial() {
        return RANDOM.nextBoolean() ? String.valueOf((char) (RANDOM.nextInt(26) + 'a')) : null;
    }

    private static java.sql.Date randomDateOfBirth() {
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.YEAR, 1975 + RANDOM.nextInt(50));
        calendar.set(Calendar.MONTH, RANDOM.nextInt(12));
        calendar.set(Calendar.DAY_OF_MONTH, RANDOM.nextInt(28) + 1);
        return new java.sql.Date(calendar.getTimeInMillis());
    }

    public static String randomAddress() {
        int civicNumber = RANDOM.nextInt(1000) + 1; // Random number between 1 and 1000
        String streetName = STREET_NAMES[RANDOM.nextInt(STREET_NAMES.length)]; // Random street name

        return civicNumber + " " + streetName + " Street";
    }

    private static String randomGender() {
        return GENDERS[RANDOM.nextInt(GENDERS.length)];
    }

    private static String randomCities() {
        return CITIES[RANDOM.nextInt(CITIES.length)];
    }

    private static String randomHobbies() {
        return HOBBIES[RANDOM.nextInt(HOBBIES.length)];
    }

    private static String randomSSN() {
        return (RANDOM.nextInt(999) + 1) + "-" +
                (RANDOM.nextInt(999) + 1) + "-" +
                (RANDOM.nextInt(999) + 1);
    }

    public static String randomPostalCode() {

        // Generate first segment (letter digit letter)

        return String.valueOf((char) (RANDOM.nextInt(26) + 'A')) +
                RANDOM.nextInt(10) +
                (char) (RANDOM.nextInt(26) + 'A') +
                " " +

                // Generate second segment (digit letter digit)
                RANDOM.nextInt(10) +
                (char) (RANDOM.nextInt(26) + 'A') +
                RANDOM.nextInt(10);
    }
}
