CREATE DATABASE assignment1;

USE assignment1;

CREATE TABLE Donors
(
    donorID       INT         NOT NULL,
    firstName     VARCHAR(50) NOT NULL,
    lastName      VARCHAR(50) NOT NULL,
    middleInitial CHAR(1),
    dateOfBirth   DATE,
    address       VARCHAR(100), -- might need to change to civic code only
    city          VARCHAR(50),
    postalCode    VARCHAR(7),
    province      ENUM (
        'AB',                   -- Alberta
        'BC',                   -- British Columbia
        'MB',                   -- Manitoba
        'NB',                   -- New Brunswick
        'NL',                   -- Newfoundland and Labrador
        'NS',                   -- Nova Scotia
        'NT',                   -- Northwest Territories
        'NU',                   -- Nunavut
        'ON',                   -- Ontario
        'PE',                   -- Prince Edward Island
        'QC',                   -- Quebec
        'SK',                   -- Saskatchewan
        'YT'                    -- Yukon
        ),
    gender        ENUM ('M', 'F', 'U'),
    SSN           VARCHAR(11),
    hobby         VARCHAR(100),
    phone         VARCHAR(15),
    email         VARCHAR(100),
    CONSTRAINT donors_pk PRIMARY KEY (donorID)
);

CREATE TABLE Donations
(
    dID     INT            NOT NULL,
    donorID INT            NOT NULL,
    date    DATE           NOT NULL,
    type    VARCHAR(7)     NOT NULL,
    amount  DECIMAL(10, 2) NOT NULL,
    CONSTRAINT donations_pk PRIMARY KEY (dID),
    CONSTRAINT donations_donorID_fk FOREIGN KEY (donorID) REFERENCES Donors (donorID),
    CONSTRAINT valid_donation_type CHECK (type IN ('money', 'product'))
);

create table Products
(
    pID         INT            NOT NULL,
    description varchar(100),
    date        DATE           NOT NULL,
    price       DECIMAL(10, 2) NOT NULL,
    weight      DECIMAL(5, 2)  NOT NULL,
    CONSTRAINT products_pk PRIMARY KEY (pID)
);

CREATE TABLE Sales
(
    sID         INT                      NOT NULL,
    date        DATE                     NOT NULL,
    amount      DECIMAL(10, 2)           NOT NULL,
    totalWeight DECIMAL(10, 2)           NOT NULL,
    deliveryFee DECIMAL(10, 2) DEFAULT 0 NOT NULL,
    CONSTRAINT sales_pk primary key (sID),
    CONSTRAINT valid_delivery_fee CHECK (deliveryFee >= 0)
);

CREATE TABLE SalesItems
(
    sID INT NOT NULL,
    pID INT NOT NULL,
    PRIMARY KEY (sID, pID),
    FOREIGN KEY (sID) REFERENCES Sales (sID),
    FOREIGN KEY (pID) REFERENCES Products (pID)
);
